(function (global) {
    "use strict";
    //reupload test
    global.FRCC = global.FRCC || {};
    global.FRCC.Portal = global.FRCC.Portal || {};
    global.FRCC.Portal.Utilities = (function () {
        class EygaApiType {
            static Documents = "eyfrcc_documentapiurl";
            static Identity = "eyfrcc_identityapiurl";
            static Address = "eyfrcc_addressapiurl";
            static SAMGov = "eyfrcc_samgovapiurl";
            static SAML = "eyfrcc_samlapiurl";
        }

        class EygaApiContext {
            constructor(regardingId, regardingType, userId) {
                this.regardingId = regardingId ? regardingId.replace("{", "").replace("}", "") : null;
                this.regardingType = regardingType;
                this.userId = userId ? userId.replace("{", "").replace("}", "") : null;
            }
        }

        class EygaApiCall {
            constructor(displayName, apiType, apiMethod, httpMethod, xhrFields, contentType, processData) {
                this.displayName = displayName;
                this.apiType = apiType;
                this.apiMethod = apiMethod;
                this.httpMethod = httpMethod;
                this.xhrFields = xhrFields != undefined ? xhrFields : {};
                this.contentType = contentType != undefined ? contentType : "application/json";
                this.processData = processData != undefined ? processData : true;
            }
        }

        class ApiCall {
            static GetSASUrl = new EygaApiCall("Get SAS Url", EygaApiType.Documents, "/getsasurl", "GET", false);
            static UploadDocument = new EygaApiCall("Upload Document", EygaApiType.Documents, "/upload", "POST", {}, false, false);
            static DownloadDocument = new EygaApiCall("Download Document", EygaApiType.Documents, "/download", "POST", { responseType: "blob" });
            static DeleteDocument = new EygaApiCall("Delete Download", EygaApiType.Documents, "/delete", "DELETE");
            static GetDocumentsById = new EygaApiCall("Get Documents By Id", EygaApiType.Documents, "/searchbytags", "POST");
            static IdentityRequest = new EygaApiCall("Initiate Identity Request", EygaApiType.Identity, "/requests", "POST");
            static ValidateAddress = new EygaApiCall("Validate Address", EygaApiType.Address, "/validate", "POST");
            static GetSAMGovEntity = new EygaApiCall("Get SAM.gov Entity", EygaApiType.SAMGov, "/entities", "GET");
            static GetSAMGovExclusion = new EygaApiCall("Get SAM.gov Exclusion", EygaApiType.SAMGov, "/exclusions", "GET");
            static GetSAMLToken = new EygaApiCall("Get SAML Token", EygaApiType.SAML, "/logout", "POST");
        }

        class Dictionary {
            constructor() {
                this._results = {};
            }

            add = (identifier, result) => (this._results[identifier] = result);
            get = (identifier) => this._results[identifier];
            remove = (identifier) => delete this._results[identifier];
        }

        const StepNames = Object.freeze({
            RiskIndicator: "Risk Indicator",
        });

        const RiskAssessmentOutcomeType = Object.freeze({
            AddFieldToReviewChecklist: 643_260_000,
            AddFieldToApplication: 643_260_001,
            AddStageToReviewProcess: 643_260_002,
            AdjustReportingFrequency: 643_260_003,
            DenyApplication: 643_260_004,
        });

        const invalidAddressMessage = "Please enter a valid address.";

        var saveSession = function (name, obj) {
            sessionStorage.setItem(name, JSON.stringify(obj));
            return true;
        };

        var getSession = function (name) {
            var obj = {};
            var itemFromStorage = sessionStorage.getItem(name);
            if (itemFromStorage !== "undefined") {
                obj = JSON.parse(itemFromStorage);
            }
            return obj;
        };

        // Given the ID of a step on an application, "opens" the div for that step, swapping the images appropriately
        var openAccordian = function (id) {
            var image = $(`#${id}-image`); // Remove after all application forms have been published
            var accordianContent = $(`#${id}`);
            var accordianHeader = $(`#${id}-accordion`); // Remove after all application forms have been published
            var accordianHeaderButton = $(`#${id}-accordion button`);
            var accordianHeaderImage = $(`#${id}-accordion i`);

            if (accordianHeader.hasClass("disabled") || accordianHeaderButton.prop("disabled")) {
                return;
            }

            if (accordianContent.hasClass("hidden")) {
                accordianHeaderImage.removeClass("glyphicon-menu-right");
                accordianHeaderImage.addClass("glyphicon-menu-down");
                accordianContent.removeClass("hidden");
                accordianHeaderButton.attr("aria-expanded", true);
                image.attr("src", "/Images/down-arrow.svg"); // Remove after all application forms have been published
            } else {
                accordianHeaderImage.removeClass("glyphicon-menu-down");
                accordianHeaderImage.addClass("glyphicon-menu-right");
                accordianContent.addClass("hidden");
                accordianHeaderButton.attr("aria-expanded", false);
                image.attr("src", "/Images/side-arrow.svg"); // Remove after all application forms have been published
            }
        };

        var handleTablePaging = function (
            previousPage,
            currentPage,
            serviceUrl,
            selectorPrefix,
            createRowCallback,
            setOnClickEvent,
            finalCallback = null,
            tableNumber = null
        ) {
            resetTablePaging(selectorPrefix);

            const requestUrl = serviceUrl.replace("{0}", currentPage);
            appAjax("Retrieving records...", {
                type: "GET",
                url: requestUrl,
                contentType: "application/json",
                success: async function (response) {
                    const data = JSON.parse(response);
                    if (data.results.length > 0) {
                        createRowCallback.constructor.name === "AsyncFunction"
                            ? await _refreshTable(data.results, selectorPrefix, createRowCallback)
                            : _refreshTable(data.results, selectorPrefix, createRowCallback);
                        _refreshPageList(currentPage, data.itemsPerPageCount, data, setOnClickEvent, selectorPrefix, tableNumber);
                    } else {
                        handleTablePagingNoResults(selectorPrefix);
                        currentPage = previousPage;
                    }

                    if (finalCallback) {
                        finalCallback(data.results, data?.totalRecordCount);
                    }

                    hideContextualProgressIndicator(selectorPrefix);
                },
                failure: function (res) {
                    currentPage = previousPage;
                },
            });

            return currentPage;
        };

        const resetTablePaging = function (selectorPrefix) {
            showContextualProgressIndicator(selectorPrefix);

            const noResultsMessage = `.${selectorPrefix}NoResultsMessage`;
            $(noResultsMessage).hide();

            $(`#${selectorPrefix}Next`).addClass("disabled");
            $(`#${selectorPrefix}Previous`).addClass("disabled");
        };

        const handleTablePagingNoResults = function (selectorPrefix) {
            _displayNoResults(selectorPrefix);
            $("#page-list").empty();
            $(`#${selectorPrefix}Next`).removeClass("disabled");
            $(`#${selectorPrefix}Previous`).removeClass("disabled");

            const pageButtonList = $(`#${selectorPrefix}-page-list-custom`);
            pageButtonList.empty();
        };

        var _refreshTable = async function (results, selectorPrefix, createRowCallback) {
            //clear all rows
            $(`#${selectorPrefix}Table`).find("tbody>tr").remove();

            var callbackresults = [];
            //populate table with paginated data
            for (let i = 0; i < results.length; i++) {
                callbackresults.push(createRowCallback(results[i]));
            }

            var rows = await Promise.all(callbackresults);

            rows.forEach((rowHtml) => {
                $(`#${selectorPrefix}Table`).find("tbody").append(rowHtml);
            });

            // initialize any popover elements that were added
            $('[data-toggle="popover"]').popover();
        };

        var _refreshPageList = function (currentPage, itemsPerPage, result, setOnClickEvent, selectorPrefix, tableNumber) {
            var pageButtonList = $(`#${selectorPrefix}-page-list-custom`);
            pageButtonList.empty();

            //making this reusable for components with pagination in a for loop
            var sectionSelector;
            if (result.thirdPartyId) {
                sectionSelector = result.thirdPartyId;
            }

            if (!itemsPerPage) itemsPerPage = 10;
            var totalPages = Math.ceil(result.totalRecordCount / itemsPerPage);

            //remove pagination for 1 page
            if (totalPages == 1) {
                return;
            }

            //add left arrow - disabled if on page 1
            var previousPage = currentPage - 1;
            if (previousPage == 0) {
                pageButtonList.append(
                    `<li class="disabled"><a id="${selectorPrefix}Previous" data-page="${previousPage}" aria-label="Previous page" title="Previous Page" class="entity-pager-prev-link" data-original-title="Previous page"><span aria-hidden="true">&lt;</span></a></li>`
                );
            } else {
                pageButtonList.append(
                    `<li><a href="javascript:void(0)" id="${selectorPrefix}Previous" data-page="${previousPage}" aria-label="Previous page" title="Previous Page" class="entity-pager-prev-link" data-original-title="Previous page"><span aria-hidden="true">&lt;</span></a></li>`
                );
                pageButtonList
                    .children("li")
                    .last()
                    .on("click", function () {
                        if (sectionSelector) {
                            setOnClickEvent(previousPage, sectionSelector);
                        } else if (tableNumber != null && tableNumber >= 0) {
                            setOnClickEvent(previousPage, tableNumber);
                        } else {
                            setOnClickEvent(previousPage);
                        }
                    });
            }

            //determine range of buttons around selected page

            //scenario 1: beginning of list
            if (currentPage < 8 || (currentPage == 8 && totalPages == 8)) {
                //if there are less than 8 pages don't need ellipses or last page
                if (totalPages <= 8) {
                    _createPageLinks(1, totalPages, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                } else {
                    _createPageLinks(1, 8, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                    //is this check really necessary??
                    if (totalPages != 9) {
                        pageButtonList.append(
                            `<li class="disabled"><a aria-label="ellipsis indicating non-visible pages" data-page=".." aria-disabled="true"><span aria-hidden="true">..</span></a></li>`
                        );
                    }
                    pageButtonList.append(
                        `<li><a href="javascript:void(0)" aria-label="page ${totalPages}" data-page="${totalPages}" aria-current="false">${totalPages}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(totalPages, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(totalPages, tableNumber);
                            } else {
                                setOnClickEvent(totalPages);
                            }
                        });
                }
            }
            //display first page and ellipses for following two scenarios
            else {
                //display page 1
                pageButtonList.append(`<li><a href="javascript:void(0)" aria-label="page 1" data-page="1" aria-current="false">1</a></li>`);
                pageButtonList
                    .children("li")
                    .last()
                    .on("click", function () {
                        if (sectionSelector) {
                            setOnClickEvent(1, sectionSelector);
                        } else if (tableNumber != null && tableNumber >= 0) {
                            setOnClickEvent(1, tableNumber);
                        } else {
                            setOnClickEvent(1);
                        }
                    });
                //display ellipses
                pageButtonList.append(
                    `<li class="disabled"><a aria-label="ellipsis indicating non-visible pages" data-page=".." aria-disabled="true"><span aria-hidden="true">..</span></a></li>`
                );

                //scenario 2: showing end of list
                if (currentPage > totalPages - 7) {
                    _createPageLinks(totalPages - 7, totalPages, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                }
                //scenario 3: show middle of list
                else {
                    _createPageLinks(currentPage - 4, currentPage + 4, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber);
                    //display ellipses
                    pageButtonList.append(
                        `<li class="disabled"><a aria-label="ellipsis indicating non-visible pages" data-page=".." aria-disabled="true"><span aria-hidden="true">..</span></a></li>`
                    );
                    //display last page
                    pageButtonList.append(
                        `<li><a href="javascript:void(0)" aria-label="page ${totalPages}" data-page="${totalPages}" aria-current="false">${totalPages}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(totalPages, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(totalPages, tableNumber);
                            } else {
                                setOnClickEvent(totalPages);
                            }
                        });
                }
            }

            //add next arrow - disable if no more records
            var nextPage = currentPage + 1;
            if (nextPage <= totalPages || result.moreRecords) {
                //display extra pages if reached query limit but still additional pages
                if (currentPage == totalPages && result.moreRecords) {
                    pageButtonList.append(
                        `<li><a href="javascript:void(0)" aria-label="page ${nextPage}" data-page="${nextPage}" aria-current="false">${nextPage}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(nextPage, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(nextPage, tableNumber);
                            } else {
                                setOnClickEvent(nextPage);
                            }
                        });
                }

                pageButtonList.append(
                    `<li><a href="javascript:void(0)" id="${selectorPrefix}Next" data-page="${nextPage}" aria-label="Next page" title="Next Page" class="entity-pager-next-link" data-original-title="Next page"><span aria-hidden="true">&gt;</span></a></li>`
                );
                pageButtonList
                    .children("li")
                    .last()
                    .on("click", function () {
                        if (sectionSelector) {
                            setOnClickEvent(nextPage, sectionSelector);
                        } else if (tableNumber != null && tableNumber >= 0) {
                            setOnClickEvent(nextPage, tableNumber);
                        } else {
                            setOnClickEvent(nextPage);
                        }
                    });
            } else {
                pageButtonList.append(
                    `<li class="disabled"><a id="${selectorPrefix}Next" data-page="${nextPage}" aria-label="Next page" title="Next Page" class="entity-pager-next-link" data-original-title="Next page"><span aria-hidden="true">&gt;</span></a></li>`
                );
            }
            if (currentPage != 1) {
                if ($(`#${selectorPrefix}TableContainer a[data-page="${currentPage}"]`).length !== 0) {
                    $(`#${selectorPrefix}TableContainer a[data-page="${currentPage}"]`).focus();
                    return;
                }

                // TO-DO: remove after all custom tables have been updated with _TableContainer
                $("a[data-page=" + currentPage + "]").focus();
            }
        };

        var _createPageLinks = function (startIndex, endIndex, currentPage, setOnClickEvent, selectorPrefix, sectionSelector, tableNumber) {
            var pageButtonList = $(`#${selectorPrefix}-page-list-custom`);

            for (var i = startIndex; i <= endIndex; i++) {
                (function (buttonNum) {
                    pageButtonList.append(
                        `<li class="${
                            buttonNum == currentPage ? "active" : ""
                        }"><a href="javascript:void(0)" aria-label="page ${buttonNum}" data-page="${buttonNum}" aria-current="${
                            buttonNum == currentPage ? "page" : "false"
                        }">${buttonNum}</a></li>`
                    );
                    pageButtonList
                        .children("li")
                        .last()
                        .on("click", function () {
                            if (sectionSelector) {
                                setOnClickEvent(buttonNum, sectionSelector);
                            } else if (tableNumber != null && tableNumber >= 0) {
                                setOnClickEvent(buttonNum, tableNumber);
                            } else {
                                setOnClickEvent(buttonNum);
                            }
                        });
                })(i);
            }
        };

        var _displayNoResults = function (selectorPrefix) {
            $(`#${selectorPrefix}Table`).find("tbody>tr").remove();
            $(`.${selectorPrefix}NoResultsMessage`).show();
        };

        var _showHidePageButtons = function (moreRecords, currentPage, selectorPrefix) {
            if (!moreRecords) {
                _disablePageButton(`#${selectorPrefix}Next`);
            } else {
                _enablePageButton(`#${selectorPrefix}Next`);
            }

            if (currentPage == 1) {
                _disablePageButton(`#${selectorPrefix}Previous`);
            } else {
                _enablePageButton(`#${selectorPrefix}Previous`);
            }

            if (!moreRecords && currentPage === 1) {
                _disablePageButton(`#${selectorPrefix}PageCount`);
            } else {
                _enablePageButton(`#${selectorPrefix}PageCount`);
            }
        };

        var _enablePageButton = function (buttonSelector) {
            $(buttonSelector).parent().show();
        };

        var _disablePageButton = function (buttonSelector) {
            $(buttonSelector).parent().hide();
        };

        const updateOrderingConfiguration = function (element) {
            console.log("in updateOrderingConfiguration");
            const $element = $(element);

            const orderOn = $element.attr("data-attribute");
            if (!orderOn) {
                return;
            }

            const orderBy = $element.parent().hasClass("sort-asc") ? "descending" : "ascending";

            return {
                OrderOn: orderOn,
                OrderBy: orderBy,
            };
        };

        const clearTableSorting = function ($tableColumn) {
            $tableColumn.parents("tr").children("th").removeClass("sort sort-asc sort-desc");
            $tableColumn.parents("tr").children("th").removeAttr("aria-sort");
            $tableColumn.parents("tr").find("span.fa").remove();
            $tableColumn.parents("tr").find("span.sort-hint").remove();
        };

        const resetTableSorting = function (tableIdentifier, orderingConfiguration) {
            $(`#${tableIdentifier}TableContainer`).css("min-height", "0px");

            if (!orderingConfiguration?.OrderOn) {
                return;
            }

            const $a = $(`table#${tableIdentifier}Table th.sort-enabled a[data-attribute="${orderingConfiguration.OrderOn}"]`);

            clearTableSorting($a);

            const orderBy = orderingConfiguration?.OrderBy || "ascending";
            const orderByClassName = !orderBy ? "sort-asc" : orderBy === "ascending" ? "sort-asc" : "sort-desc";
            $a.parent().addClass(`sort ${orderByClassName}`);
            $a.parent().attr("aria-sort", orderBy);

            const sortingIconClassName = orderByClassName === "sort-asc" ? "fa-arrow-up" : "fa-arrow-down";
            $a.append(`<span class="fa ${sortingIconClassName}" aria-hidden="true"></span>`);
            $a.append(`<span class="sr-only sort-hint">. sort ${orderBy}</span>`);
        };

        const handleTableSorting = function (target, tableSettingsConfiguration) {
            const $parentTable = $(target).parents("table");
            const tableIdentifier = $parentTable.attr("data-ordering");
            if (!tableIdentifier) {
                return;
            }

            const tableSettings = tableSettingsConfiguration[tableIdentifier];
            tableSettings.OrderingConfiguration = updateOrderingConfiguration(target);
            if (tableSettings.DynamicCount == null || tableSettings.DynamicCount < 0) {
                tableSettings.PagingCallback(1);
            } else {
                tableSettings.PagingCallback(1, tableSettings.DynamicCount);
            }
        };

        var formatDate = function (date) {
            if (!date || date === "") {
                return date;
            }
            const monthNames = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
            const day = date.substr(3, 2);
            const month = monthNames[Number(date.substr(0, 2)) - 1];
            const year = date.substr(6, 4);
            return `${month} ${day}, ${year}`;
        };

        var searchForError = function (nameKey, myArray) {
            if (myArray && myArray.Blob) {
                for (var i = 0; i < myArray.Blob.length; i++) {
                    if (myArray.Blob[i].Name["#text"] === nameKey) {
                        return true;
                    }
                }
                if (myArray && !Array.isArray(myArray.Blob)) {
                    if (myArray.Blob.Name["#text"] === nameKey) {
                        return true;
                    }
                }
            }
            return false;
        };

        var _safeAjax = function safeAjax(ajaxOptions) {
            var deferredAjax = $.Deferred();
            shell
                .getTokenDeferred()
                .done(function (token) {
                    // Add headers for ajax
                    if (!ajaxOptions.headers) {
                        $.extend(ajaxOptions, {
                            headers: {
                                __RequestVerificationToken: token,
                            },
                        });
                    } else {
                        ajaxOptions.headers["__RequestVerificationToken"] = token;
                    }
                    $.ajax(ajaxOptions)
                        .done(function (data, textStatus, jqXHR) {
                            validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                        })
                        .fail(deferredAjax.reject); //ajax
                })
                .fail(function () {
                    deferredAjax.rejectWith(this, arguments); // On token failure pass the token ajax and args
                });
            return deferredAjax.promise();
        };

        var appAjax = function (processingMsg, ajaxOptions) {
            console.log(processingMsg);
            return _safeAjax(ajaxOptions)
                .fail(function (response) {
                    if (response.responseJSON) {
                        if (response.responseJSON.error.innererror) {
                            openConfirmationModal("Error", response.responseJSON.error.innererror.message, "OK", "Cancel", true, true, false, false, true);
                        } else {
                            openConfirmationModal("Error", response.responseJSON.error.message, "OK", "Cancel", true, true, false, false, true);
                        }
                    } else {
                        openConfirmationModal("Error", "Web API is not available...", "OK", "Cancel", true, true, false, false, true);
                    }
                })
                .always(FRCC.Portal.Notification.hide);
        };

        const asyncAppAjax = async function (ajaxOptions, skipSettingCache = true) {
            // Retrieve data from cache (if available)
            const cachedData = retrieveAPIResponseFromCache(ajaxOptions.url, ajaxOptions.type);

            // If cached data was retrieved, serve immediately while making background call to API to refresh cache
            if (cachedData) {
                makeAjaxRequest(ajaxOptions, skipSettingCache);
                return $.Deferred().resolve(cachedData).promise();
            } else {
                return makeAjaxRequest(ajaxOptions, skipSettingCache);
            }
        };

        const makeAjaxRequest = async function (ajaxOptions, skipSettingCache) {
            const deferredAjax = $.Deferred();

            shell
                .getTokenDeferred()
                .done(function (token) {
                    if (!ajaxOptions.headers) {
                        $.extend(ajaxOptions, {
                            headers: {
                                __RequestVerificationToken: token,
                            },
                        });
                    } else {
                        ajaxOptions.headers["__RequestVerificationToken"] = token;
                    }
                    $.ajax(ajaxOptions)
                        .done(function (data, textStatus, jqXHR) {
                            populateAPIResponseCache(ajaxOptions.url, ajaxOptions.type, data, skipSettingCache);
                            validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                        })
                        .fail(deferredAjax.reject);
                })
                .fail(function () {
                    deferredAjax.rejectWith(this, arguments);
                });

            return deferredAjax.promise();
        };

        const populateAPIResponseCache = async function (key, requestType, data, skipSettingCache) {
            if (skipSettingCache) {
                return;
            }

            if (requestType !== "GET") {
                return;
            }

            const cacheDuration = await getEYGAConfigurationSetting("eyfrcc_cacheduration");
            if (cacheDuration == 0) {
                return;
            }

            const expires = Date.now() + cacheDuration * 60 * 1000;
            const cacheEntry = { data: data, expires: expires };

            try {
                localStorage.setItem(key, JSON.stringify(cacheEntry));
            } catch (error) {
                console.error(`Error occurred setting localStorage: ${error}`);
            }
        };

        const retrieveAPIResponseFromCache = function (key, requestType) {
            if (requestType !== "GET") {
                return null;
            }

            const cachedEntryObject = localStorage.getItem(key);
            if (!cachedEntryObject) {
                return null;
            }

            const cachedEntry = JSON.parse(cachedEntryObject);
            if (Date.now() > cachedEntry.expires) {
                localStorage.removeItem(key);
                return null;
            }

            return cachedEntry.data;
        };

        var _isOldIE = function (userAgent) {
            var msie = userAgent.indexOf("MSIE");
            var trident = userAgent.indexOf("Trident/");
            if (msie > 0 || trident > 0) {
                return true; // IE 10 or older
            }
            return false; // other browser, IE 11, or Edge
        };

        var showIEWarning = function () {
            if (_isOldIE(navigator.userAgent)) {
                var ieWarning = document.getElementById("ie-banner");
                if (ieWarning) {
                    ieWarning.setAttribute("style", "display: block;");
                }
            }
        };

        var _generateExpirationDate = function () {
            var date = new Date();
            date.setDate(date.getDate() + 1);
            return date;
        };

        var _generateCurrentDatePlusOneMinute = function () {
            var date = new Date();
            var oneMinuteInMilliSeconds = 60000;
            date = new Date(date.getTime() + oneMinuteInMilliSeconds);
            return date;
        };

        var getEYGAConfigurationSetting = async (settingName) => {
            var refreshSetting = true;
            var configurationSetting = window.sessionStorage.getItem(settingName);

            if (configurationSetting) {
                configurationSetting = JSON.parse(configurationSetting);
                var currentDatePlusOneMinute = _generateCurrentDatePlusOneMinute();
                if (currentDatePlusOneMinute.getTime() <= new Date(configurationSetting.expirationDate).getTime()) {
                    refreshSetting = false;
                }
            }

            if (refreshSetting) {
                var settingValue = await _getEYGAConfigurationSettingFromCrm(settingName);
                sessionStorage.setItem(settingName, JSON.stringify(settingValue));
            }

            configurationSetting = JSON.parse(window.sessionStorage.getItem(settingName));
            return configurationSetting.value[0][settingName];
        };

        var _getEYGAConfigurationSettingFromCrm = function (settingName) {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "GET",
                    url: `/_api/eyfrcc_eygaconfigurations?$select=eyfrcc_eygaconfigurationid,eyfrcc_name,${settingName}`,
                    contentType: "application/json",
                    success: function (result, status, xhr) {
                        if (!result || !result.value || result.value.length < 1 || !result.value[0][settingName]) {
                            return null;
                        }

                        result.expirationDate = _generateExpirationDate();
                        resolve(result);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    },
                });
            });
        };

        var _getApiToken = function () {
            return new Promise(function (resolve, reject) {
                var apiTokenKey = "apiToken";
                _getApiTokenFromApi()
                    .then((value) => {
                        window.sessionStorage.setItem(apiTokenKey, JSON.stringify(value));
                        resolve(value.eyfrcc_token);
                    })
                    .catch((reason) => reject(reason));
            });
        };

        var _getApiTokenFromApi = function () {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "GET",
                    url: `/_api/eyfrcc_apitokens(${_generateGuid()})?$select=eyfrcc_token,eyfrcc_expirationdate`,
                    contentType: "application/json",
                    success: function (res, status, xhr) {
                        resolve(res);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    },
                });
            });
        };

        var _generateGuid = function () {
            var randomGuid = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx".replace(/x/g, function (c) {
                var r = (Math.random() * 16) | 0,
                    v = c === "x" ? r : (r & 0x3) | 0x8;
                return v.toString(16);
            });
            return randomGuid;
        };

        var callEygaApi = async (apiCall, apiContext, payload, queryParams = null) => {
            var results = await Promise.all([getEYGAConfigurationSetting(apiCall.apiType), getHeaders(apiCall.displayName, apiContext)]);

            if (!results[0] || !results[1]) {
                return null; // Error messages for retrieval failure handled within respective calls
            }

            return new Promise(function (resolve, reject) {
                $.ajax({
                    type: apiCall.httpMethod,
                    url: results[0] + apiCall.apiMethod + (queryParams ? queryParams : ""),
                    headers: results[1],
                    data: apiCall.contentType == "application/json" ? JSON.stringify(payload) : payload,
                    contentType: apiCall.contentType,
                    processData: apiCall.processData,
                    xhrFields: apiCall.xhrFields,
                    success: function (response, status, request) {
                        resolve(response);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        handleError(error, true);
                        reject(error);
                    },
                });
            });
        };

        var getHeaders = function (apiAction, apiContext, generateEventId = true) {
            return Promise.all([
                _getApiToken(),
                _createEvent(generateEventId, apiAction, apiContext.regardingId, apiContext.regardingType, apiContext.userId),
                getEYGAConfigurationSetting("eyfrcc_eygaapikey"),
            ])
                .then((result) => {
                    if (!result || result.length < 1) {
                        throw new Error("ERROR: Unexpected error retrieving API call headers");
                    }
                    if (!result[0] || result[0] == "") {
                        throw new Error("ERROR: Unexpected error retrieving API token");
                    }
                    if (!result[2] || result[2] == "") {
                        throw new Error("ERROR: Could not retrieve EYGA API Key; check that a valid key has been entered in the configuration");
                    }
                    return _returnHeaders(result[2], result[1], apiContext.regardingId, result[0]);
                })
                .catch((error) => handleError(error));
        };

        var handleError = function (errorMessage, displayError = false) {
            if (displayError) {
                const modalErrorMessage = errorMessage ? errorMessage : "An unexpected error occurred. Please try again in a moment.";
                openConfirmationModal("Error", modalErrorMessage, "OK", "Cancel", true, true, false);
            }

            console.log(errorMessage);
        };

        var handleAjaxError = function (request, status, error) {
            console.log(error);
        };

        var downloadPortalPageDocument = function (portalPageDocumentId) {
            var documentRecordUrl = `/_api/eyfrcc_portalpagedocuments(${portalPageDocumentId})/eyfrcc_document/$value`;
            window.open(documentRecordUrl, "_self");
        };

        var _returnHeaders = function (apiKey, eventId, regardingId, apiToken) {
            if (eventId) {
                return {
                    "eyga-api-key": apiKey,
                    "EYGA-EventId": eventId,
                    "EYGA-RecordId": regardingId,
                    "EYGA-Authorization": apiToken,
                };
            }

            if (regardingId) {
                return {
                    "eyga-api-key": apiKey,
                    "EYGA-RecordId": regardingId,
                    "EYGA-Authorization": apiToken,
                };
            }

            if (!apiToken || apiToken == "") {
                handleError("ERROR: An error occured attempting to retrieve the API token", true);
                return null;
            }

            if (!apiKey || apiKey == "") {
                handleError("ERROR: An error occured attempting to retrieve the API key; check if key is present in the configuration settings", true);
                return null;
            }

            return { "eyga-api-key": apiKey, "EYGA-Authorization": apiToken };
        };

        var _createEvent = function (generateEventId, apiAction, regardingId, regardingType, userId) {
            if (generateEventId && regardingId && regardingType && userId) {
                var event = {};
                event["eyfrcc_RegardingId_" + regardingType + "@odata.bind"] = `/${getEntitySetName(regardingType)}(${regardingId})`;
                event["eyfrcc_PortalUser@odata.bind"] = `/contacts(${userId})`;
                event["eyfrcc_name"] = apiAction;
                event["eyfrcc_source"] = 643260000; // Portal

                return new Promise(function (resolve, reject) {
                    _safeAjax({
                        type: "POST",
                        url: "/_api/eyfrcc_events",
                        contentType: "application/json",
                        data: JSON.stringify(event),
                        success: function (res, status, xhr) {
                            var eventId = xhr.getResponseHeader("entityid");
                            resolve(eventId);
                        },
                        error: function (request, status, error) {
                            console.log("ERROR: Event creation for " + apiAction + " failed");
                            handleAjaxError(request, status, error);
                            resolve(); // Return nothing if event creation fails, but continue API call
                        },
                    });
                });
            } else {
                Promise.resolve();
            }
        };

        var getEntitySetName = (logicalName) => {
            if (!logicalName) {
                return logicalName;
            }
            if (logicalName.endsWith("s")) {
                return logicalName + "es";
            }
            if (logicalName.endsWith("y")) {
                const entitySetName = `${logicalName.slice(0, -1)}ies`;
                return entitySetName;
            }
            return logicalName + "s";
        };

        var handleRiskAssessmentOutcomes = async () => {
            var urlParams = new URLSearchParams(window.location.search);
            var parentAppId = urlParams.get("parentApplicationId");
            var childAppId = urlParams.get("formId");

            var outcomes = await _getRiskAssessmentOutcomes(childAppId, parentAppId);

            _handleAddFieldToApplicationOutcomes(outcomes);
            // Add code for any new logic for risk assessment outcomes here
        };

        var _handleAddFieldToApplicationOutcomes = function (outcomes) {
            var accordion = $(".accordion-title:contains('" + StepNames.RiskIndicator + "')").closest(".accordion");
            if (!accordion.length || !accordion.attr("id")) {
                return;
            }

            var contentId = accordion.attr("id").slice(0, -10);

            outcomes
                .filter((outcome) => outcome.eyfrcc_outcometype === RiskAssessmentOutcomeType.AddFieldToApplication && outcome.outcomefieldlogicalname)
                .forEach((outcome) => {
                    var row = $(["#", contentId, " #", outcome.outcomefieldlogicalname].join("")).closest("tr");
                    if (!row) {
                        return;
                    }

                    if (outcome.applicationoutcomeid) {
                        row.removeClass("hidden");
                    } else {
                        row.addClass("hidden");
                    }
                });
        };

        var _getRiskAssessmentOutcomes = function (childAppId, parentAppid) {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "GET",
                    url: ["/Services/GetRiskAssessmentOutcomesByApplication/?formId=", childAppId, "&parentApplicationId=", parentAppid].join(""),
                    contentType: "application/json",
                    success: function (res, status, xhr) {
                        res = JSON.parse(res);
                        resolve(res ? res.results : res);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    },
                });
            });
        };

        var showProgressIndicator = (message) => {
            $("p#progressIndicatorText").text(message || "Loading...");
            $("div#progressIndicator").addClass("visible");

            // accessibility fix: prevent users from navigating around the screen with tab/scrolling while loading events are occurring
            $(document).on("keydown", trapFocusKeydownEvent);
            document.body.style.marginRight = "15px";
            document.body.style.overflowY = "hidden";
        };

        var hideProgressIndicator = () => {
            $("div#progressIndicator").removeClass("visible");

            // return the ability to scroll/tab once loading event is completed
            $(document).off("keydown", trapFocusKeydownEvent);
            document.body.style.marginRight = "";
            document.body.style.overflowY = "";
        };

        const trapFocusKeydownEvent = function (e) {
            const TabKey = 9;
            if (e.keyCode === TabKey) {
                e.preventDefault();
            }
        };

        var showContextualProgressIndicator = (identifier, message) => {
            $(`#${identifier}LoadingIndicatorText`).text(message || "Loading...");
            $(`#${identifier}LoadingIndicator`).addClass("visible");
        };

        var hideContextualProgressIndicator = (identifier) => {
            $(`#${identifier}LoadingIndicator`).removeClass("visible").parent().css("min-height", "0px");
        };

        var viewMessage = async function (id, messagesData, regardingId, regardingType, userId) {
            console.warn(
                "DEPRECATION WARNING: viewMessage is deprecated and will be removed in future versions. Use FRCC.AmeriCorps.Modals.Messages.openViewMessageModal instead."
            );

            FRCC.AmeriCorps.Modals.Messages.openViewMessageModal(id, messagesData, regardingId, regardingType, userId);
        };

        var showCreateMessageModal = async function (regardingId, regardingType, userId, subject, isReplying = false) {
            console.warn(
                "DEPRECATION WARNING: showCreateMessageModal is deprecated and will be removed in future versions. Use FRCC.AmeriCorps.Modals.Messages.openCreateMessageModal instead."
            );

            FRCC.AmeriCorps.Modals.Messages.openCreateMessageModal(regardingId, regardingType, userId, subject, isReplying);
        };

        var openConfirmationModal = function (
            title = "Confirm",
            message = "Are you sure?",
            confirmLabel = "Confirm",
            cancelLabel = "Cancel",
            showAlert = false,
            showAlertDanger = false,
            showCancelOption = true,
            moveToTop = false,
            addHelpContactInfoToError = false
        ) {
            return new Promise(function (resolve, reject) {
                const overwriteMessage = overwriteErrorMessage(message, addHelpContactInfoToError);
                if (overwriteMessage !== null && !overwriteMessage.isHtml) {
                    message = overwriteMessage.updatedMessage;
                }

                var confirmModalErrorContainerSelector = "#eyga-confirm-modal .modal-body .modal-body-error-container";

                $("#eyga-confirm-modal").attr("aria-label", title);
                $("#eyga-confirm-modal .modal-title").text(title);

                if (showAlert) {
                    $(confirmModalErrorContainerSelector).show();
                    $(`${confirmModalErrorContainerSelector} .modal-body-error-description span`).remove(); // Remove any previous html contained within modal-body-description

                    if (overwriteMessage !== null && overwriteMessage.isHtml) {
                        $(`${confirmModalErrorContainerSelector} .modal-body-error-description`).text(null);
                        $(`${confirmModalErrorContainerSelector} .modal-body-error-description`).append(overwriteMessage.updatedMessage);
                    } else {
                        $(`${confirmModalErrorContainerSelector} .modal-body-error-description`).text(message);
                    }

                    $("#eyga-confirm-modal .modal-body .modal-body-description").hide();

                    showAlertDanger
                        ? $(confirmModalErrorContainerSelector).addClass("alert-danger").removeClass("alert-warning")
                        : $(confirmModalErrorContainerSelector).removeClass("alert-danger").addClass("alert-warning");
                } else {
                    $(confirmModalErrorContainerSelector).hide();
                    $("#eyga-confirm-modal .modal-body .modal-body-description").show();

                    $("#eyga-confirm-modal .modal-body .modal-body-description").text(message);
                }

                if (moveToTop) {
                    $("#eyga-confirm-modal").css("z-index", "9999999");
                }

                showCancelOption ? $("button.cancel").show() : $("button.cancel").hide();

                const confirmModal = $("#eyga-confirm-modal");
                const confirmButton = $("#eyga-confirm-modal button.primary");
                const closeButtons = $("#eyga-confirm-modal button.cancel, #eyga-confirm-modal button.form-close");

                confirmButton.text(confirmLabel);
                confirmButton.click(() => {
                    confirmModal.modal("hide");
                    confirmButton.off("click");
                    closeButtons.off("click");
                    resolve(true);
                });

                $("#eyga-confirm-modal button.cancel").text(cancelLabel);
                closeButtons.click(() => {
                    confirmModal.modal("hide");
                    confirmButton.off("click");
                    closeButtons.off("click");
                    resolve(false);
                });

                confirmModal.modal();
            });
        };

        var openCancellationModal = function (applicationId, redirectToApplications = false) {
            $(".modal-body-error-container").hide();
            $("#eyga-cancel-application-modal").modal("show");

            $(".cancel-application-button")
                .off("click")
                .on("click", function () {
                    _cancelApplication(applicationId, redirectToApplications);
                });
        };

        var openUpdateProfileModal = function (userId, userFirstName, userMiddleName, userLastName, userSuffix, userEmailAddress, userTelephone, userJobTitle) {
            _loadProfileFields(userFirstName, userMiddleName, userLastName, userSuffix, userEmailAddress, userTelephone, userJobTitle);
            $(".modal-body-error-container").hide();
            $("#eyga-update-profile-modal").modal();

            $("#eyga-update-profile-submit")
                .off("click")
                .on("click", async function () {
                    if (!(await _validateModalFields("#eyga-update-profile-modal"))) {
                        return;
                    }

                    var firstname = $("#eyga-update-profile-firstname").val();
                    var middlename = $("#eyga-update-profile-middlename").val();
                    var lastname = $("#eyga-update-profile-lastname").val();
                    var suffix = $("#eyga-update-profile-suffix").val();
                    var emailAddress = $("#eyga-update-profile-emailaddress").val();
                    var telephone = $("#eyga-update-profile-telephone").val();
                    var jobTitle = $("#eyga-update-profile-jobtitle").val();

                    var payload = JSON.stringify({
                        ["firstname"]: firstname,
                        ["lastname"]: lastname,
                        ["middlename"]: middlename,
                        ["eyfrcc_suffix"]: suffix,
                        ["emailaddress1"]: emailAddress,
                        ["telephone1"]: telephone,
                        ["jobtitle"]: jobTitle,
                    });

                    _updateProfile(userId, payload);
                });
        };

        var _loadProfileFields = function (userFirstName, userMiddleName, userLastName, userSuffix, userEmailAddress, userTelephone, userJobTitle) {
            $("#eyga-update-profile-firstname").val(userFirstName);
            $("#eyga-update-profile-middlename").val(userMiddleName);
            $("#eyga-update-profile-lastname").val(userLastName);
            $("#eyga-update-profile-suffix").val(userSuffix);
            $("#eyga-update-profile-emailaddress").val(userEmailAddress);
            $("#eyga-update-profile-telephone").val(userTelephone);
            $("#eyga-update-profile-jobtitle").val(userJobTitle);
        };

        var openUpdateProjectModal = function (userId, organizationId, id, name, stateId, programId, address1, address2, city, zipCode, activities) {
            _loadProjectFields(name, stateId, programId, address1, address2, city, zipCode, activities);
            $("#eyga-create-project-modal").on("hidden.bs.modal", (e) => {
                $(`#eyga-create-project-modal.modal-body`).scrollTop(0);
            });

            $(".modal-body-error-container").hide();
            $("#eyga-create-project-modal").modal();

            $("#eyga-create-project-state-territory")
                .off("change")
                .on("change", (e) => {
                    const $state = $(e.currentTarget);
                    const $program = $("#eyga-create-project-program");
                    FRCC.AmeriCorps.Modals.Project.checkProjectState(organizationId, programId || $program.val(), $state.val());
                });

            $("#eyga-create-project-save-draft")
                .off("click")
                .on("click", function () {
                    _updateProject(id, true, userId);
                });

            $("#eyga-create-project-submit")
                .off("click")
                .on("click", function () {
                    _updateProject(id, false, userId);
                });
        };

        var _loadProjectFields = function (name, stateId, programId, address1, address2, city, zipCode, activities) {
            $("#eyga-create-project-program").val(programId);
            $("#eyga-create-project-name").val(name);
            $("#eyga-create-project-activities").val(activities);
            $("#eyga-create-project-street-address-1").val(address1);
            $("#eyga-create-project-street-address-2").val(address2);
            $("#eyga-create-project-city").val(city);
            $("#eyga-create-project-state-territory").val(stateId);
            $("#eyga-create-project-zip-code").val(zipCode);
        };

        var _updateProject = async function (id, isDraft, userId) {
            const zipcode = $("#eyga-create-project-zip-code").val();
            const customValidations = [
                {
                    TargetField: "#eyga-create-project-street-address-1",
                    EvaluationFunction: async () => {
                        return await FRCC.AmeriCorps.Modals.AddressValidation.validateAddress(userId, {
                            StreetAddress1: "#eyga-create-project-street-address-1",
                            StreetAddress2: "#eyga-create-project-street-address-2",
                            City: "#eyga-create-project-city",
                            State: "#eyga-create-project-state-territory",
                            PostalCode: "#eyga-create-project-zip-code",
                        });
                    },
                    Message: invalidAddressMessage,
                },
                {
                    TargetField: "#eyga-create-project-zip-code",
                    EvaluationFunction: () => {
                        return FRCC.AmeriCorps.Modals.Validations.validateZipCode(zipcode);
                    },
                    Message: "Please enter a valid USPS 9-digit zip code or select the suggested address from the Confirm Address validation.",
                },
            ];

            if (!isDraft) {
                if (!(await _validateModalFields("#eyga-create-project-modal", true, customValidations))) {
                    return;
                }
            }

            var program = $("#eyga-create-project-program").val();
            var name = $("#eyga-create-project-name").val();
            var activities = $("#eyga-create-project-activities").val();
            var address1 = $("#eyga-create-project-street-address-1").val();
            var address2 = $("#eyga-create-project-street-address-2").val();
            var city = $("#eyga-create-project-city").val();
            var state = $("#eyga-create-project-state-territory").val();

            var requireReview = (function getRequireReview() {
                var yesNo;
                $.ajax({
                    type: "Get",
                    url: `/_api/eyfrcc_programs(${program})`,
                    async: false,
                    contentType: "application/json",
                    success: function (response) {
                        yesNo = response.eyfrcc_requirenewprojectreview;
                    },
                });
                return yesNo;
            })();

            var payloadObj = {
                ["eyfrcc_name"]: name,
                ["eyfrcc_projectactivities"]: activities ? activities : "",
                ["eyfrcc_projectaddressstreet1"]: address1,
                ["eyfrcc_projectaddressstreet2"]: address2,
                ["eyfrcc_projectaddresscity"]: city,
                ["eyfrcc_projectaddresszippostalcode"]: zipcode,
                ["eyfrcc_projectstatus"]: isDraft ? 643260005 : requireReview ? 643260000 : 643260001,
            };

            if (program) {
                payloadObj["eyfrcc_program@odata.bind"] = `/eyfrcc_programs(${program})`;
            }

            if (state) {
                payloadObj["eyfrcc_projectaddressstateterritory@odata.bind"] = `/eyfrcc_stateterritories(${state})`;
            }

            _patchProject(id, JSON.stringify(payloadObj), true);
        };

        var _patchProject = function (id, payload, displayAlert = false) {
            _ajaxRequest_Modal(
                {
                    type: "PATCH",
                    url: `/_api/eyfrcc_projects(${id})`,
                    data: payload,
                    success: function (res) {
                        console.log(res);
                        console.log("in success for update project on profile page");
                        handleProjectsPage(1);

                        hideProgressIndicator();
                        $("#eyga-create-project-modal").modal("hide");
                        if (displayAlert) displaySuccessAlert("Project has been updated");
                    },
                },
                "Updating Project..."
            );
        };

        var openUpdateOrganizationModal = async function (create, orgDetail) {
            var samGovEnabled = orgDetail.samGovEnabled;
            var userId = orgDetail.userId;
            var organizationId = orgDetail.organizationId;
            var hasSubmittedApp = false;

            const appResponse = await asyncAppAjax({
                type: "GET",
                url: `/Services/GetAppsFromOrg/?orgIdF=${organizationId}`,
                contentType: "application/json",
                success: async function (response) {
                    const data = JSON.parse(response);
                    if (data.results.length > 0) {
                        console.log(`Length: ${data.results.length}`);
                        hasSubmittedApp = true;
                    }
                },
                failure: function (res) {
                    console.log("error");
                },
            });

            const modalTitle = create ? "Create Organization" : "Update Organization";
            $("#eyga-update-organization-modal .modal-title").text(modalTitle);

            if (!create) {
                _loadOrganizationFields(orgDetail, hasSubmittedApp);
            } else {
                $("#uei-number-yes").prop("checked", true);
                $("#update-organization-has-uei").show();
                $("#update-organization-no-uei").hide();
            }

            if (!samGovEnabled) {
                $(".uei-yes-no-container").hide();
                $("#update-organization-has-uei").hide();
                $("#update-organization-no-uei").show();
                $("#uei-number-no").prop("checked", true);
            }

            _clearValidationErrors();
            $("#eyga-update-organization-modal").modal();

            $("#eyga-update-organization-submit")
                .off("click")
                .on("click", function () {
                    handleUpdateOrganizationSubmit(orgDetail, userId, organizationId, create);
                });

            $('input[name="uei-number"]')
                .off("change")
                .on("change", function () {
                    if ($('input[name="uei-number"]:checked').val() === "true") {
                        $("#update-organization-has-uei").show();
                        $("#update-organization-no-uei").hide();
                    } else {
                        $("#update-organization-has-uei").hide();
                        $("#update-organization-no-uei").show();
                    }
                });
        };

        var handleUpdateOrganizationSubmit = async function (orgDetail, userId, organizationId, create) {
            const indicatorLabel = create ? "Creating Organization..." : "Updating Organization...";

            const customValidations = [
                {
                    TargetField: "#eyga-update-organization-address-street1",
                    EvaluationFunction: async () => {
                        return await FRCC.AmeriCorps.Modals.AddressValidation.validateAddress(userId, {
                            StreetAddress1: "#eyga-update-organization-address-street1",
                            StreetAddress2: "#eyga-update-organization-address-street2",
                            City: "#eyga-update-organization-address-city",
                            State: "#eyga-update-organization-address-state",
                            PostalCode: "#eyga-update-organization-address-postalcode",
                        });
                    },
                    Message: invalidAddressMessage,
                },
                {
                    TargetField: "#eyga-update-organization-address-postalcode",
                    EvaluationFunction: () => {
                        return FRCC.AmeriCorps.Modals.Validations.validateZipCode($("#eyga-update-organization-address-postalcode").val());
                    },
                    Message: "Zip Code must be in the following format: #####-####",
                },
                {
                    TargetField: "#eyga-update-organization-address2-street1",
                    EvaluationFunction: async () => {
                        return await FRCC.AmeriCorps.Modals.AddressValidation.validateAddress(userId, {
                            StreetAddress1: "#eyga-update-organization-address2-street1",
                            StreetAddress2: "#eyga-update-organization-address2-street2",
                            City: "#eyga-update-organization-address2-city",
                            State: "#eyga-update-organization-address2-state",
                            PostalCode: "#eyga-update-organization-address2-postalcode",
                        });
                    },
                    Message: invalidAddressMessage,
                },
                {
                    TargetField: "#eyga-update-organization-address2-postalcode",
                    EvaluationFunction: () => {
                        return FRCC.AmeriCorps.Modals.Validations.validateZipCode($("#eyga-update-organization-address2-postalcode").val());
                    },
                    Message: "Zip Code must be in the following format: #####-####",
                },
            ];

            var selector = $('input[name="uei-number"]:checked').val() === "false" ? "#update-organization-no-uei" : "#update-organization-has-uei";
            var hasUniqueIdentifier = $('input[name="uei-number"]:checked').val() === "true";
            if (!(await _validateModalFields(selector, true, hasUniqueIdentifier ? [] : customValidations))) {
                return;
            }

            if (hasUniqueIdentifier) {
                showProgressIndicator(indicatorLabel);
                _handleOrganizationUpdate_SAMGov(orgDetail, userId, organizationId, create, indicatorLabel);
                return;
            }

            if (!_validateOrganizationFields()) {
                $("#eyga-update-organization-modal .modal-body").scrollTop(0);
                return;
            }

            _handleOrganizationUpdate_Manual(orgDetail, userId, organizationId, create);
        };

        var _handleOrganizationUpdate_SAMGov = function (orgDetail, userId, organizationId, create) {
            const uniqueIdentifier = $("#eyga-update-organization-uniqueidentifier").val();
            _safeAjax({
                type: "GET",
                url: `/_api/accounts?$filter=eyfrcc_uniqueidentifier eq '${uniqueIdentifier}'`,
                contentType: "application/json",
                success: async function (response, status, xhr) {
                    if (response.value.length !== 0) {
                        _handleModalError(
                            null,
                            '<span>This UEI number has already been assigned to another organization. Please contact the support team at <span style="color: red;text-decoration: underline;">support@americorps.gov</span> or <span style="font-weight: bold;">800-555-1234</span> to be added to this organization.</span>'
                        );
                        return;
                    }

                    var eygaApiContext = new EygaApiContext(userId, "contact", userId);
                    var response;
                    try {
                        response = await callEygaApi(ApiCall.GetSAMGovEntity, eygaApiContext, null, `?id=${uniqueIdentifier}`);
                    } catch (ex) {
                        hideProgressIndicator();
                        $("#eyga-update-organization-modal").modal("hide");
                        return;
                    }

                    var confirmation =
                        !create && response?.entityData.length > 0 ? await _confirmChangeToOrganization(orgDetail, response.entityData[0]) : true;
                    if (!confirmation) {
                        return;
                    }

                    var payload = JSON.stringify({
                        ["eyfrcc_hasuniqueidentifier"]: true,
                        ["eyfrcc_uniqueidentifier"]: uniqueIdentifier,
                        ["name"]: "(No name)",
                        ["eyfrcc_lastdatarefresh"]: _getUtcDateNow(),
                    });
                    if (create) {
                        _createOrganization(userId, payload);
                    } else {
                        _updateOrganization(organizationId, payload);
                    }
                },
                error: function (request, status, error) {
                    _handleModalError(request.responseJSON.error.innererror.message);
                    handleAjaxError(request, status, error);
                },
            });
        };

        var _handleOrganizationUpdate_Manual = function (orgDetail, userId, organizationId, create) {
            var employerIdentificationNumber = $("#eyga-update-organization-employeridentificationnumber").val();
            var name = $("#eyga-update-organization-name").val();

            const primaryBusinessTypeId = $("#eyga-update-organization-primary-business-type").val();

            var street1 = $("#eyga-update-organization-address-street1").val();
            var street2 = $("#eyga-update-organization-address-street2").val();
            var city = $("#eyga-update-organization-address-city").val();
            var stateTerritoryId = $("#eyga-update-organization-address-state").val();
            var postalCode = $("#eyga-update-organization-address-postalcode").val();

            var street1Physical = $("#eyga-update-organization-address2-street1").val();
            var street2Physical = $("#eyga-update-organization-address2-street2").val();
            var cityPhysical = $("#eyga-update-organization-address2-city").val();
            var stateTerritoryIdPhysical = $("#eyga-update-organization-address2-state").val();
            var postalCodePhysical = $("#eyga-update-organization-address2-postalcode").val();

            var telephone = $("#eyga-update-organization-telephone").val();
            var extension = $("#eyga-update-organization-extension").val();

            var payload = JSON.stringify({
                ["eyfrcc_hasuniqueidentifier"]: false,
                ["eyfrcc_uniqueidentifier"]: null,
                ["eyfrcc_taxpayeridentificationnumber"]: employerIdentificationNumber,
                ["name"]: name,
                ["eyfrcc_OrganizationalPrimaryBusinessTypeId@odata.bind"]: `/eyfrcc_businesstypes(${primaryBusinessTypeId})`,
                ["address2_line1"]: street1Physical,
                ["address2_line2"]: street2Physical,
                ["address2_city"]: cityPhysical,
                ["eyfrcc_Address2StateTerritoryId@odata.bind"]: `/eyfrcc_stateterritories(${stateTerritoryIdPhysical})`,
                ["address2_postalcode"]: postalCodePhysical,
                ["address1_line1"]: street1,
                ["address1_line2"]: street2,
                ["address1_city"]: city,
                ["eyfrcc_StateProvinceId@odata.bind"]: `/eyfrcc_stateterritories(${stateTerritoryId})`,
                ["address1_postalcode"]: postalCode,
                ["telephone1"]: telephone,
                ["eyfrcc_phoneextension"]: extension,
            });

            if (create) {
                _createOrganization(userId, payload);
                return;
            }

            _updateOrganization(organizationId, payload);
        };

        var _confirmChangeToOrganization = async function (orgDetail, entityData) {
            _initializeConfirmUEIChangesModal(orgDetail, entityData);

            hideProgressIndicator();
            return new Promise(function (resolve, reject) {
                const $modal = $("#eyga-confirm-uei-changes-modal");
                const $confirmButton = $("#eyga-confirm-uei-changes-submit");
                const $closeButton = $("#eyga-confirm-uei-changes-cancel");

                $confirmButton.off("click").on("click", () => {
                    resolve(true);
                });

                $closeButton.off("click").on("click", () => {
                    resolve(false);
                });

                $modal.modal("show");
            });
        };

        var _initializeConfirmUEIChangesModal = function (orgDetail, entityData) {
            const modalSelector = "#eyga-confirm-uei-changes-modal";
            const defaultTextValue = "—";

            _setOrganizationDataText(`${modalSelector} [data-attribute="organization-name"]`, orgDetail.orgName, defaultTextValue);
            _setOrganizationDataText(`${modalSelector} [data-attribute="organization-type"]`, orgDetail.orgType, defaultTextValue);
            _setOrganizationDataText(`${modalSelector} [data-attribute="organization-ein"]`, orgDetail.orgEmployerIdentificationNumber, defaultTextValue);
            _setOrganizationDataText(`${modalSelector} [data-attribute="organization-telephone"]`, orgDetail.orgTelephone, defaultTextValue);
            _setOrganizationDataText(`${modalSelector} [data-attribute="organization-uei"]`, orgDetail.orgUniqueIdentifier, defaultTextValue);

            _setSAMGovDataText(
                `${modalSelector} [data-attribute="samgov-name"]`,
                orgDetail.orgName,
                entityData.entityRegistration.legalBusinessName,
                defaultTextValue
            );
            _setSAMGovDataText(
                `${modalSelector} [data-attribute="samgov-type"]`,
                orgDetail.orgType,
                entityData.coreData.generalInformation.entityTypeDesc,
                defaultTextValue
            );
            _setSAMGovDataText(
                `${modalSelector} [data-attribute="samgov-ein"]`,
                orgDetail.orgEmployerIdentificationNumber,
                entityData?.coreData?.tinInformation?.taxpayerIdentificationNumber,
                defaultTextValue
            );
            _setSAMGovDataText(
                `${modalSelector} [data-attribute="samgov-telephone"]`,
                orgDetail.orgTelephone,
                entityData?.pointsOfContact?.governmentBusinessPOC?.usPhone,
                defaultTextValue
            );
            _setSAMGovDataText(
                `${modalSelector} [data-attribute="samgov-uei"]`,
                orgDetail.orgUniqueIdentifier,
                entityData.entityRegistration.ueiSAM,
                defaultTextValue
            );
        };

        const _setOrganizationDataText = function (selector, value, defaultValue) {
            $(selector).text(value !== "" ? value : defaultValue);
        };

        const _setSAMGovDataText = function (selector, currentValue, samGovValue, defaultValue) {
            const value = samGovValue === null || samGovValue === "" ? defaultValue : samGovValue;
            const formattedValue = _formatSAMGovData(currentValue, value);

            $(selector).html(formattedValue);
        };

        const _formatSAMGovData = function (originalValue, samGovValue) {
            if (!samGovValue) {
                return samGovValue;
            }

            const isDifferent = samGovValue !== originalValue;

            return isDifferent ? `<em class="text-danger text-weight-semibold">${samGovValue}</em>` : samGovValue;
        };

        var _validateOrganizationFields = function () {
            _clearValidationErrors();

            var ein = $("#eyga-update-organization-employeridentificationnumber").val();
            if (ein === "") {
                return true;
            }

            var einPattern = /^\d{9}$/;
            if (!einPattern.test(ein)) {
                $("#eyga-update-organization-employeridentificationnumber").parent().addClass("has-error");
                _handleModalError(
                    "",
                    '<a href="#eyga-update-organization-employeridentificationnumber">Employer Identification Number (EIN)</a> must be 9 digits.'
                );
                return false;
            }

            var phoneNumber = $("#eyga-update-organization-telephone").val();
            if (phoneNumber === "") {
                return true;
            }

            var numbersOnlyPattern = /^\d+$/;
            if (!numbersOnlyPattern.test(phoneNumber)) {
                $("#eyga-update-organization-telephone").parent().addClass("has-error");
                _handleModalError("", '<a href="#eyga-update-organization-telephone">Phone</a> must contain only digits.');
                return false;
            }

            var extension = $("#eyga-update-organization-extension").val();
            if (extension === "") {
                return true;
            }

            if (!numbersOnlyPattern.test(extension)) {
                $("#eyga-update-organization-extension").parent().addClass("has-error");
                _handleModalError("", '<a href="#eyga-update-organization-extension">Extension</a> must contain only digits.');
                return false;
            }

            return true;
        };

        var _clearValidationErrors = function () {
            $(".has-error").removeClass("has-error");
            $(".modal-body-error-container").hide();
        };

        var _loadOrganizationFields = function (orgDetail, hasSubmittedApp) {
            if (orgDetail.orgHasUniqueIdentifier) {
                $("#uei-number-yes").prop("checked", true);
                $("#update-organization-has-uei").show();
                $("#update-organization-no-uei").hide();
            } else {
                $("#uei-number-no").prop("checked", true);
                $("#update-organization-has-uei").hide();
                $("#update-organization-no-uei").show();

                if (hasSubmittedApp) {
                    $("#eyga-update-organization-unqiueidentifier").prop("readonly", true);
                    $("#eyga-update-organization-employeridentificationnumber").prop("readonly", true);
                    $("#eyga-update-organization-name").prop("readonly", true);
                    $("#eyga-update-organization-primary-business-type").prop("disabled", true);
                    $("#eyga-update-organization-businesstype")
                        .prop({
                            readonly: true,
                        })
                        .css({
                            "pointer-events": "none",
                        })
                        .addClass("disabled")
                        .off("click");
                    $("#eyga-update-organization-address-street1").prop("readonly", true);
                    $("#eyga-update-organization-address-street2").prop("readonly", true);
                    $("#eyga-update-organization-address-city").prop("readonly", true);
                    $("#eyga-update-organization-address-state").prop("disabled", true);
                    $("#eyga-update-organization-address-postalcode").prop("readonly", true);
                    $("#eyga-update-organization-address2-street1").prop("readonly", true);
                    $("#eyga-update-organization-address2-street2").prop("readonly", true);
                    $("#eyga-update-organization-address2-city").prop("readonly", true);
                    $("#eyga-update-organization-address2-state").prop("disabled", true);
                    $("#eyga-update-organization-address2-postalcode").prop("readonly", true);
                    $("#eyga-update-organization-telephone").prop("readonly", true);
                    $("#eyga-update-organization-extension").prop("readonly", true);
                }
            }

            $("#eyga-update-organization-unqiueidentifier").val(orgDetail.orgUniqueIdentifier);
            $("#eyga-update-organization-employeridentificationnumber").val(orgDetail.orgEmployerIdentificationNumber);
            $("#eyga-update-organization-name").val(orgDetail.orgName);
            $("#eyga-update-organization-primary-business-type").val(orgDetail.orgPrimaryBusinessTypeId);
            orgDetail.orgBusinessTypes.forEach((businessType) => {
                $("#eyga-update-organization-businesstype").next().find(`li[aria-selected="false"]:contains("${businessType.Name}")`).click();
            });
            $("#eyga-update-organization-address-street1").val(orgDetail.orgMailingStreet1);
            $("#eyga-update-organization-address-street2").val(orgDetail.orgMailingStreet2);
            $("#eyga-update-organization-address-city").val(orgDetail.orgMailingCity);
            $("#eyga-update-organization-address-state").val(orgDetail.orgMailingState);
            $("#eyga-update-organization-address-postalcode").val(orgDetail.orgMailingPostalCode);
            $("#eyga-update-organization-address2-street1").val(orgDetail.orgPhysicalStreet1);
            $("#eyga-update-organization-address2-street2").val(orgDetail.orgPhysicalStreet2);
            $("#eyga-update-organization-address2-city").val(orgDetail.orgPhysicalCity);
            $("#eyga-update-organization-address2-state").val(orgDetail.orgPhysicalState);
            $("#eyga-update-organization-address2-postalcode").val(orgDetail.orgPhysicalPostalCode);
            $("#eyga-update-organization-telephone").val(orgDetail.orgTelephone);
            $("#eyga-update-organization-extension").val(orgDetail.orgExtension);
        };

        var _getUtcDateNow = function () {
            var utcDate = new Date(
                Date.UTC(
                    new Date().getUTCFullYear(),
                    new Date().getUTCMonth(),
                    new Date().getUTCDate(),
                    new Date().getUTCHours(),
                    new Date().getUTCMinutes(),
                    new Date().getUTCSeconds(),
                    new Date().getUTCMilliseconds()
                )
            );

            return utcDate;
        };

        var _getApplicationTableFields = function (entityName) {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "GET",
                    url: `/Services/GetApplicationTableFields/?entityname=${entityName}`,
                    contentType: "application/json",
                    success: function (res, status, xhr) {
                        res = JSON.parse(res);
                        resolve(res);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    },
                });
            });
        };

        var openUpsertAppTableRecordModal = async function (modalId, entityName, options, existingRecordId) {
            // Reset values
            $(`#${modalId} input.eyga-app-table-field`).val(null);
            $(`#${modalId} select.eyga-app-table-field`).each(() => {
                $(this).val($(this).find("option:first").val());
            });
            // Clear any errors
            $(".modal-body-error-container").hide();

            if (options?.modalTitle) {
                $(`#${modalId} .modal-title`).text(options.modalTitle || "Create Record");
            }

            if (options?.passingValues) {
                $(`#${modalId} #TotalProposedBudget`).text(parseFloat(options?.passingValues?.tp || 0).toFixed(2));
                $(`#${modalId} #TotalMSYRequested`).text(parseFloat(options?.passingValues?.msy || 0).toFixed(2));
            }

            var recordValues = options?.recordValues || {};

            $(`#${modalId} .eyga-app-table-record-submit`)
                .off("click")
                .on("click", async function () {
                    if (!(await _validateModalFields(`#${modalId}`))) {
                        return;
                    }

                    $(`#${modalId} .eyga-app-table-field`).each(function () {
                        var fieldIdentifier = $(this).data("apptablefield");
                        if (this.type == "number") {
                            let v = parseFloat($(this).val());
                            recordValues[fieldIdentifier] = v;
                        } else {
                            recordValues[fieldIdentifier] = $(this).val();
                        }
                    });

                    await _upsertAppTableRecord(
                        entityName,
                        recordValues,
                        options?.progressMessage,
                        options?.successCallback,
                        existingRecordId,
                        options?.entitySetName
                    );
                });

            $(`#${modalId}`).modal("show");
        };

        var _convertAppTableRecordToEntity = async function (entityName, record) {
            var appTableFields = await _getApplicationTableFields(entityName);
            var entity = {};
            for (var key of Object.keys(record)) {
                var identifierStart = key.indexOf("{");
                var configIdentifier = identifierStart >= 0 ? key.substring(identifierStart + 1, key.indexOf("}")) : key;
                if (!appTableFields[configIdentifier]) {
                    continue;
                }

                var entityPropertyName = appTableFields[configIdentifier];
                entityPropertyName = key.replace("{", "").replace("}", "").replace(configIdentifier, entityPropertyName);
                entity[entityPropertyName] = record[key] ? record[key] : null;
            }

            return entity;
        };

        var _upsertAppTableRecord = function (entityName, record, progressMessage, successCallback, existingRecordId, entitySetName) {
            showProgressIndicator(progressMessage || "Saving...");

            if (!entitySetName) {
                entitySetName = getEntitySetName(entityName);
            }

            return _convertAppTableRecordToEntity(entityName, record)
                .then((entity) => {
                    _safeAjax({
                        type: existingRecordId ? "PATCH" : "POST",
                        url: existingRecordId ? `/_api/${entitySetName}(${existingRecordId})` : `/_api/${entitySetName}`,
                        data: JSON.stringify(entity),
                        contentType: "application/json",
                        success: function () {
                            hideProgressIndicator();
                            $(".modal").modal("hide");
                            if (successCallback) {
                                successCallback();
                            }
                        },
                        error: function (request, status, error) {
                            _handleModalError(request.responseJSON.error.innererror.message);
                            handleAjaxError(request, status, error);
                        },
                    });
                })
                .catch((error) => {
                    _handleModalError(error);
                    handleAjaxError(error);
                });
        };

        var openCreateOrganizationInvitationModal = function (userId, organizationId) {
            $("#eyga-create-organization-invitation-firstname").val(null);
            $("#eyga-create-organization-invitation-lastname").val(null);
            $("#eyga-create-organization-invitation-emailaddress").val(null);
            $("#eyga-create-organization-invitation-contacttype").val($("#eyga-create-organization-invitation-contacttype option:first").val());
            $(".modal-body-error-container").hide();
            $("#eyga-create-organization-invitation-modal").modal();

            const customValidations = [
                {
                    TargetField: "#eyga-create-organization-invitation-emailaddress",
                    EvaluationFunction: async () => {
                        return await FRCC.AmeriCorps.Modals.Validations.validateEmailAddress($("#eyga-create-organization-invitation-emailaddress").val());
                    },
                    Message: "Please enter a valid email address",
                },
            ];

            $("#eyga-create-organization-invitation-submit")
                .off("click")
                .on("click", async function () {
                    if (!(await _validateModalFields("#eyga-create-organization-invitation-modal", true, customValidations))) {
                        return;
                    }

                    var firstname = $("#eyga-create-organization-invitation-firstname").val();
                    var lastname = $("#eyga-create-organization-invitation-lastname").val();
                    var emailAddress = $("#eyga-create-organization-invitation-emailaddress").val();
                    var organizationContactType = $("#eyga-create-organization-invitation-contacttype").val();

                    var payload = JSON.stringify({
                        ["eyfrcc_firstname"]: firstname,
                        ["eyfrcc_lastname"]: lastname,
                        ["eyfrcc_emailaddress"]: emailAddress,
                        ["eyfrcc_organizationcontacttype"]: organizationContactType,
                        ["eyfrcc_Organization@odata.bind"]: `/accounts(${organizationId})`,
                        ["eyfrcc_From@odata.bind"]: `/contacts(${userId})`,
                    });

                    _createOrganizationInvitation(payload, true);
                });
        };

        var openCreateProjectModal = function (organizationId, userId, isApplicationView = false, programId = null, callbackFunc) {
            if (!isApplicationView) {
                $("#eyga-create-project-program").val($("#eyga-create-project-program option:first").val());
                $("#eyga-create-project-save-draft")
                    .off("click")
                    .on("click", function () {
                        _saveProject(organizationId, userId, true, isApplicationView, programId, callbackFunc);
                    });
            } else {
                $("#eyga-create-project-modal").on("hidden.bs.modal", (e) => {
                    $("#eyga-create-project-modal.modal-body").scrollTop(0);
                    $(".eyfrcc_fakedropdown").find('option[value="0"]').attr("selected", true);
                });
            }

            $("#eyga-create-project-name").val(null);
            $("#eyga-create-project-for-planning-grant").val($("#eyga-create-project-for-planning-grant option:first").val());
            $("#eyga-create-project-activities").val(null);
            $("#eyga-create-project-street-address-1").val(null);
            $("#eyga-create-project-street-address-2").val(null);
            $("#eyga-create-project-city").val(null);
            $("#eyga-create-project-state-territory").val($("#eyga-create-project-state-territory option:first").val());
            $("#eyga-create-project-zip-code").val(null);

            $(".modal-body-error-container").hide();
            $("#eyga-create-project-modal").modal();

            $("#eyga-create-project-submit")
                .off("click")
                .on("click", function () {
                    _saveProject(organizationId, userId, false, isApplicationView, programId, callbackFunc);
                });

            $("#eyga-create-project-state-territory")
                .off("change")
                .on("change", (e) => {
                    const $state = $(e.currentTarget);
                    const $program = $("#eyga-create-project-program");
                    FRCC.AmeriCorps.Modals.Project.checkProjectState(organizationId, programId || $program.val(), $state.val());
                });
        };

        var openUpdateUserTypeModal = function (contactId, contactName, contactType) {
            $("#eyga-update-user-type-contacttype").val(contactType);
            $("#eyga-update-user-type-modal .modal-body .modal-body-content").text(contactName);
            $(".modal-body-error-container").hide();
            $("#eyga-update-user-type-modal").modal();

            $("#eyga-update-user-type-submit")
                .off("click")
                .on("click", function () {
                    var organizationContactType = $("#eyga-update-user-type-contacttype").val();

                    var payload = JSON.stringify({
                        ["eyfrcc_organizationcontacttype"]: organizationContactType,
                    });

                    _updateContactUserType(contactId, payload, true);
                });
        };

        const _validateModalFields = async function (modalSelector, validateRequiredFields = true, customValidations = []) {
            _clearValidationErrors();
            let isValid = true;
            let validationCancelled = false;

            let customValidationsHtml = [];
            for (const customValidation of customValidations) {
                const isFieldValid = await customValidation.EvaluationFunction(); // Assume this might return a promise
                if (isFieldValid === null) {
                    validationCancelled = true;
                    break;
                } else if (isFieldValid === false) {
                    isValid = false;
                    customValidationsHtml.push(`<li><a href="${customValidation.TargetField}">${customValidation.Message}</a></li>`);
                    $(customValidation.TargetField).closest(".form-group").addClass("has-error");
                }
            }

            let requiredValidationsHtml = [];
            if (validateRequiredFields) {
                const $requiredFields = $(`${modalSelector} label.required`).next("input, textarea, select").filter(":visible");
                $requiredFields.each(function () {
                    const $field = $(this);
                    if (!$field.val() || !$field.val().trim()) {
                        isValid = false;
                        const fieldId = $field.attr("id");
                        const fieldLabel = $(`label[for="${fieldId}"]`).text();
                        requiredValidationsHtml.push(`<li><a href="#${fieldId}">'${fieldLabel}' is a required field.</a></li>`);
                        $field.closest(".form-group").addClass("has-error");
                    }
                });
            }

            if (validationCancelled) {
                $(`${modalSelector} .form-group.has-error`).removeClass("has-error");
                return null;
            }

            if (!isValid) {
                const validationsHtml = requiredValidationsHtml.concat(customValidationsHtml);
                const html = `<ul>${validationsHtml.join("")}</ul>`;
                _handleModalError(null, html);
                $(`${modalSelector} .modal-body`).scrollTop(0);
            }

            return isValid;
        };

        var _cancelApplication = function (applicationId, redirectToApplications) {
            var options = {
                type: "PATCH",
                url: `/_api/eyfrcc_applications(${applicationId})`,
                data: JSON.stringify({
                    ["eyfrcc_status"]: 203300010,
                    ["eyfrcc_applicationopen"]: false,
                }),
            };

            if (redirectToApplications) {
                options.success = function () {
                    window.location.href = "/applications";
                };
            }

            _ajaxRequest_Modal(options, "Cancelling Application...");
        };

        var _updateProfile = function (userId, payload) {
            _ajaxRequest_Modal(
                {
                    type: "PATCH",
                    url: `/_api/contacts(${userId})`,
                    data: payload,
                },
                "Updating Profile..."
            );
        };

        var _createOrganization = function (userId, payload) {
            _ajaxRequest_Modal(
                {
                    type: "POST",
                    url: "/_api/accounts",
                    data: payload,
                    success: async function (res, status, xhr) {
                        var organizationId = xhr.getResponseHeader("entityid");

                        _ajaxRequest_Modal({
                            type: "PATCH",
                            url: `/_api/contacts(${userId})`,
                            data: JSON.stringify({
                                ["parentcustomerid_account@odata.bind"]: `/accounts(${organizationId})`,
                                ["eyfrcc_organizationcontacttype"]: 643260000,
                            }),
                        });
                    },
                },
                "Creating Organization..."
            );
        };

        var _updateOrganization = function (organizationId, payload) {
            _ajaxRequest_Modal(
                {
                    type: "PATCH",
                    url: `/_api/accounts(${organizationId})`,
                    data: payload,
                    success: async function (res, status, xhr) {
                        _hideModalsAndReload();
                    },
                },
                "Updating Organization..."
            );
        };

        var _createOrganizationInvitation = function (payload, displayAlert = false) {
            _ajaxRequest_Modal(
                {
                    type: "POST",
                    url: "/_api/eyfrcc_organizationinvitations",
                    data: payload,
                    success: function () {
                        handleOrgMembersPage(1);
                        hideProgressIndicator();
                        $("#eyga-create-organization-invitation-modal").modal("hide");
                        if (displayAlert) displaySuccessAlert("Invitation has been successfully sent");
                    },
                },
                "Inviting User..."
            );
        };

        var _saveProject = async function (organizationId, userId, isDraft, isApplicationView, programId, callbackFunc) {
            const customValidations = [
                {
                    TargetField: "#eyga-create-project-street-address-1",
                    EvaluationFunction: async () => {
                        return await FRCC.AmeriCorps.Modals.AddressValidation.validateAddress(userId, {
                            StreetAddress1: "#eyga-create-project-street-address-1",
                            StreetAddress2: "#eyga-create-project-street-address-2",
                            City: "#eyga-create-project-city",
                            State: "#eyga-create-project-state-territory",
                            PostalCode: "#eyga-create-project-zip-code",
                        });
                    },
                    Message: invalidAddressMessage,
                },
                {
                    TargetField: "#eyga-create-project-zip-code",
                    EvaluationFunction: () => {
                        //zip code can change when address validators run
                        return FRCC.AmeriCorps.Modals.Validations.validateZipCode($("#eyga-create-project-zip-code").val());
                    },
                    Message: "Please enter a valid USPS 9-digit zip code or select the suggested address from the Confirm Address validation. ",
                },
            ];

            if (!isDraft) {
                if (!(await _validateModalFields("#eyga-create-project-modal", true, customValidations))) {
                    return;
                }
            }

            var program = $("#eyga-create-project-program").val();
            var name = $("#eyga-create-project-name").val();
            var activities = $("#eyga-create-project-activities").val();
            var address1 = $("#eyga-create-project-street-address-1").val();
            var address2 = $("#eyga-create-project-street-address-2").val();
            var city = $("#eyga-create-project-city").val();
            var state = $("#eyga-create-project-state-territory").val();
            var forPlanningGrant = $("#eyga-create-project-for-planning-grant").val();

            var requireReview = (function getRequireReview() {
                var yesNo;
                $.ajax({
                    type: "Get",
                    url: `/_api/eyfrcc_programs(${isApplicationView ? programId : program})`,
                    async: false,
                    contentType: "application/json",
                    success: function (response) {
                        document.getElementById("eyga-create-project-modal").getElementsByClassName("modal-body")[0].scrollTop = 0;
                        yesNo = response.eyfrcc_requirenewprojectreview;
                    },
                });
                return yesNo;
            })();

            var payloadObj = {
                ["eyfrcc_name"]: name,
                ["eyfrcc_projectactivities"]: activities ? activities : "",
                ["eyfrcc_projectaddressstreet1"]: address1,
                ["eyfrcc_projectaddressstreet2"]: address2,
                ["eyfrcc_projectaddresscity"]: city,
                ["eyfrcc_projectaddresszippostalcode"]: $("#eyga-create-project-zip-code").val(),
                ["eyfrcc_OrganizationId@odata.bind"]: `/accounts(${organizationId})`,
                ["eyfrcc_projectstatus"]: isDraft ? 643260005 : requireReview ? 643260000 : 643260001,
                ["eyfrcc_forplanninggrant"]: forPlanningGrant == "0" ? false : true,
            };

            if (program) {
                payloadObj["eyfrcc_program@odata.bind"] = `/eyfrcc_programs(${program})`;
            }

            if (state) {
                payloadObj["eyfrcc_projectaddressstateterritory@odata.bind"] = `/eyfrcc_stateterritories(${state})`;
            }

            _createProject(JSON.stringify(payloadObj), requireReview, true, isApplicationView, callbackFunc);
        };

        var _createProject = function (payload, requireReview, displayAlert = false, isApplicationView, callbackFunc) {
            _ajaxRequest_Modal(
                {
                    type: "POST",
                    url: "/_api/eyfrcc_projects",
                    data: payload,
                    success: function (res, status, xhr) {
                        $("#eyga-create-project-modal").modal("hide");
                        if (!isApplicationView) {
                            handleProjectsPage(1);
                        } else {
                            var projectId = xhr.getResponseHeader("entityid");
                            callbackFunc(projectId, JSON.parse(payload).eyfrcc_name, requireReview);
                        }
                        hideProgressIndicator();
                        if (displayAlert && !isApplicationView) {
                            displaySuccessAlert("Project has been created");
                        }
                    },
                },
                "Creating Project..."
            );
        };

        var _checkRequireProjectReview = function (programId) {
            _safeAjax({
                type: "Get",
                async: false,
                url: `/_api/eyfrcc_programs(${programId})`,
                contentType: "application/json",
                success: function (response, status, xhr) {
                    return response.eyfrcc_requirenewprojectreview;
                },
            }).response;
        };

        var _updateContactUserType = function (contactId, payload, displayAlert = false) {
            _ajaxRequest_Modal(
                {
                    type: "PATCH",
                    url: `/_api/contacts(${contactId})`,
                    data: payload,
                    success: function () {
                        handleOrgMembersPage(0);
                        hideProgressIndicator();
                        $("#eyga-update-user-type-modal").modal("hide");
                        if (displayAlert) displaySuccessAlert("User type has been updated successfully");
                    },
                },
                "Updating User Type..."
            );
        };

        var handlePortalUserName = function (userId) {
            $("#eyga-update-name-modal").modal("show");

            $("#eyga-update-name-submit").click(function () {
                _updatePortalUserName(userId);
            });
        };

        var _updatePortalUserName = function (userId) {
            var firstName = $("#eyga-update-name-firstName").val();
            var lastName = $("#eyga-update-name-lastName").val();

            var payload = JSON.stringify({
                ["firstname"]: firstName,
                ["lastname"]: lastName,
            });

            _ajaxRequest_Modal(
                {
                    type: "PATCH",
                    url: `/_api/contacts(${userId})`,
                    data: payload,
                    success: async function (res, status, xhr) {
                        _hideModalsAndReload();
                    },
                },
                "Updating Contact..."
            );
        };

        var _ajaxRequest_Modal = function (options, progressMessage) {
            showProgressIndicator(progressMessage);

            _safeAjax({
                type: options.type,
                url: options.url,
                contentType: "application/json",
                data: options.data,
                success: function (response, status, xhr) {
                    if (options.success) {
                        options.success(response, status, xhr);
                        return;
                    }
                    _hideModalsAndReload();
                },
                error: function (request, status, error) {
                    const errorString = request.responseJSON.error.innererror?.message || request.responseJSON.error.message;
                    const errorMessage = parseErrorStringToMessage(errorString);
                    _handleModalError(errorMessage);
                    handleAjaxError(request, status, error);
                    if (options.error) {
                        options.error(request, status, error);
                    }
                },
            });
        };

        var _handleModalError = function (message, html = null) {
            hideProgressIndicator();

            const overwriteMessage = overwriteErrorMessage(message);
            if (overwriteMessage !== null && !overwriteMessage.isHtml) {
                message = overwriteMessage.updatedMessage;
            }

            $(".modal-body-error-container").show();
            $(".modal-body-error-container .modal-body-description span").remove(); // Remove any previous html contained within modal-body-description

            if (html != null || (overwriteMessage !== null && overwriteMessage.isHtml)) {
                $(".modal-body-error-container .modal-body-description").text(null);
                $(".modal-body-error-container .modal-body-description").append(html ?? overwriteMessage.updatedMessage);
                return;
            }

            $(".modal-body-error-container .modal-body-description").text(message);
        };

        var _hideModalsAndReload = function () {
            hideProgressIndicator();
            $(".modal").modal("hide");
            location.reload();
        };

        var getChildApplicationFromParent = function (applicationId) {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "Get",
                    async: false,
                    url: `/_api/eyfrcc_applications(${applicationId})?$select=_eyfrcc_childapplication_value,_eyfrcc_applicationtype_value`,
                    contentType: "application/json",
                    success: function (response, status, xhr) {
                        resolve(response);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    },
                });
            });
        };

        var getThirdPartyTypesForAppType = function (applyicationType) {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "Get",
                    async: false,
                    url: `/_api/eyfrcc_thirdparties?$filter=_eyfrcc_fundingopportunity_value eq '${applyicationType}'&$select=eyfrcc_thirdpartyid`,
                    contentType: "application/json",
                    success: function (response, status, xhr) {
                        resolve(response);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    },
                });
            });
        };

        const overwriteErrorMessage = function (message, addHelpContactInfoToError = false) {
            if (message == null) {
                // Use loose equality to allow empty strings to pass through
                return null;
            }

            switch (true) {
                case message.includes("Error Code: E-001068"):
                    return {
                        originalMessage: message,
                        updatedMessage: "Unable to remove user. A minimum of 2 administrators is required on the organization.",
                        isHtml: false,
                    };
                case message.includes("The provided anti-forgery token was meant for a different claims-based user than the current user"):
                    return {
                        originalMessage: message,
                        updatedMessage: "You have been signed out due to inactivity. Please sign in again and re-attempt this action.",
                        isHtml: false,
                    };
                case message.includes("No SAM.gov data found using the Unique Entity ID"):
                    return {
                        originalMessage: message,
                        updatedMessage:
                            "No SAM.gov data was found for the provided Unique Entity ID. Please ensure the provided UEI is correct and up to date on SAM.gov.",
                        isHtml: false,
                    };
                case message.includes("Internal Server Error") || message === "":
                    return {
                        originalMessage: message,
                        updatedMessage: `<span>
                                            An unexpected error has occurred. Please contact support if the issue persists.<br><br>
                                            AmeriCorps<br>
                                            <a href='tel:1-800-942-2677'>1-800-942-2677</a><br>
                                            <a href='mailto:help@americorps.gov'>help@americorps.gov</a>
                                        </span>`,
                        isHtml: true,
                    };
                case addHelpContactInfoToError:
                    return {
                        originalMessage: message,
                        updatedMessage: (message += `<span>
                                                <br><br>
                                                AmeriCorps<br>
                                                <a href='tel:1-800-942-2677'>1-800-942-2677</a><br>
                                                <a href='mailto:help@americorps.gov'>help@americorps.gov</a>
                                            </span>`),
                        isHtml: true,
                    };
                default:
                    return null;
            }
        };

        /**
         * OnLoad function for the advanced form web template. This function can be extended with additional logic in lieu of
         * modifying the web template record directly, allowing for ease of development with content overrides.
         */
        const onAdvancedFormLoad = function () {
            _fixPluginErrorAlerts();
        };

        /**
         * Fixes the plugin error alerts that are displayed at the top of the page.
         * This function moves the alerts to the top of the body container div and adds some attributes/styling to make them more accessible.
         */
        const _fixPluginErrorAlerts = function () {
            let alertNotification = $("body > div.alert-danger:first-child");
            let bodyContainerDiv = $("div.body-content > div.container");

            if (bodyContainerDiv.length > 0 && alertNotification.length > 0) {
                alertNotification.remove();
                bodyContainerDiv.prepend(alertNotification);
                alertNotification.addClass("vertically-align");
                alertNotification.attr("role", "alert");
            }
        };

        /**
         * Parses an error string and returns only the "message" part.
         * If the string is in the format:
         *   "Error Code: <code>. Message: <message>" it returns <message>.
         * Otherwise, it returns the original string.
         *
         * @param {string} errorString - The full error string to parse
         * @return {string} - The message portion if found; otherwise the entire string
         */
        const parseErrorStringToMessage = (errorString) => {
            const pattern = /Error Code:\s*(.*?)\.\s*Message:\s*(.*)/i;
            const match = pattern.exec(errorString);
            if (match) {
                // match[2] is the captured message text
                return match[2].trim();
            }

            // If no match, just return the original string
            return errorString;
        };

        return {
            openAccordian: openAccordian,
            searchForError: searchForError,
            appAjax: appAjax,
            asyncAppAjax: asyncAppAjax,
            showIEWarning: showIEWarning,
            formatDate: formatDate,
            getEYGAConfigurationSetting: getEYGAConfigurationSetting,
            getHeaders: getHeaders,
            handleError: handleError,
            handleAjaxError: handleAjaxError,
            callEygaApi: callEygaApi,
            saveSession: saveSession,
            getSession: getSession,
            handleTablePaging: handleTablePaging,
            updateOrderingConfiguration: updateOrderingConfiguration,
            resetTableSorting: resetTableSorting,
            handleTableSorting: handleTableSorting,
            downloadPortalPageDocument: downloadPortalPageDocument,
            handleRiskAssessmentOutcomes: handleRiskAssessmentOutcomes,
            ApiCall: ApiCall,
            Dictionary: Dictionary,
            EygaApiContext: EygaApiContext,
            EygaApiCall: EygaApiCall,
            showProgressIndicator: showProgressIndicator,
            hideProgressIndicator: hideProgressIndicator,
            showContextualProgressIndicator: showContextualProgressIndicator,
            hideContextualProgressIndicator: hideContextualProgressIndicator,
            openConfirmationModal: openConfirmationModal,
            openCancellationModal: openCancellationModal,
            openUpdateProfileModal: openUpdateProfileModal,
            openUpdateProjectModal: openUpdateProjectModal,
            openUpdateOrganizationModal: openUpdateOrganizationModal,
            openCreateOrganizationInvitationModal: openCreateOrganizationInvitationModal,
            openCreateProjectModal: openCreateProjectModal,
            openUpdateUserTypeModal: openUpdateUserTypeModal,
            viewMessage: viewMessage,
            showCreateMessageModal: showCreateMessageModal,
            openUpsertAppTableRecordModal: openUpsertAppTableRecordModal,
            getEntitySetName: getEntitySetName,
            getChildApplicationFromParent: getChildApplicationFromParent,
            getThirdPartyTypesForAppType: getThirdPartyTypesForAppType,
            overwriteErrorMessage: overwriteErrorMessage,
            handlePortalUserName: handlePortalUserName,
            onAdvancedFormLoad: onAdvancedFormLoad,
        };
    })();

    global.FRCC.Portal.Notification = (function () {
        var $processingMsgEl = $("#processingMsg"),
            _msg = "Processing...",
            _stack = 0,
            _endTimeout;

        var show = function (msg) {
            $processingMsgEl.text(msg || _msg);
            if (_stack === 0) {
                clearTimeout(_endTimeout);
                $processingMsgEl.show();
            }
            _stack++;
        };

        var hide = function () {
            _stack--;
            if (_stack <= 0) {
                _stack = 0;
                clearTimeout(_endTimeout);
                _endTimeout = setTimeout(function () {
                    $processingMsgEl.hide();
                }, 500);
            }
        };

        return {
            show: show,
            hide: hide,
        };
    })();
})(this);
